// Solutions to the Advent Of Code 2024.
// Copyright (C) 2025  Stefan Müller
//
// This program is free software: you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along with
// this program.  If not, see <http://www.gnu.org/licenses/>.

#pragma once

#include <bitset>
#include <forward_list>
#include <unordered_map>

#include <aoc/common/LogicGate.hpp>
#include <aoc/framework/Solver-types.hpp>

class CrossedWires
    : public Solver<int64_t, std::string>
{
public:
    CrossedWires(const int inputFileNameSuffix = 0);
    virtual const std::string getPuzzleName() const override;
    virtual const int getPuzzleDay() const override;
    virtual void processDataLine(const std::string& line) override;
    virtual void finish() override;
private:
    static constexpr char getInputWireDelimiter();
    static constexpr char getOutputWiresChar();
    bool isProcessingInputWires_;
    std::unordered_map<std::string, bool> wires_;
    std::forward_list<LogicGate> unevaluatedGates_;
    std::bitset<64> z_;
    void processInputWire(const std::string& line);
    void processLogicGate(const std::string& line);
    bool tryEvaluateLogicGate(const LogicGate& gate);
    void setOutputWire(const std::string& wire, const bool value);
};
